//*CMZ :  2.00/00 04/11/97  16.50.02  by  Victor Perev
//*CMZ :  1.03/03 25/08/97  16.25.33  by  Fons Rademakers
//*-- Author :    Fons Rademakers   01/07/97

//*KEEP,CopyRight,T=C.
/*************************************************************************
 * Copyright(c) 1995-1999, The ROOT System, All rights reserved.         *
 * Authors: Rene Brun and Fons Rademakers.                               *
 *                                                                       *
 * For the licensing terms see $ROOTSYS/AA_LICENSE.                      *
 * For the list of contributors see $ROOTSYS/AA_CREDITS.                 *
 *************************************************************************/
//*KEND.

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// TCondition                                                           //
//                                                                      //
// This class implements a condition variable. Use a condition variable //
// to signal threads. The actual work is done via the TConditionImp     //
// class (either TPosixCondition, TSolarisCondition or TNTCondition).   //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

//*KEEP,TCondition,T=C++.
#include "TCondition.h"
//*KEEP,TMutex,T=C++.
#include "TMutex.h"
//*KEEP,TThreadFactory,T=C++.
#include "TThreadFactory.h"
//*KEND.


ClassImp(TCondition)

//______________________________________________________________________________
TCondition::TCondition(TMutex *m)
{
   // Create condition variable. The actual condition implementation
   // will be provided via the TThreadFactory.
   TMutex *mm = m;

   fMutex = NULL;
   if (! mm) { fMutex = new TMutex(); mm = fMutex;}

   fConditionImp = gThreadFactory->CreateConditionImp(mm->fMutexImp);

   if (!fConditionImp)
      Error("TCondition", "could not create ConditionImp");
}


Int_t  TCondition::Wait()
{
  Int_t iret;
  if (fMutex) fMutex->Lock();
  iret = fConditionImp->Wait();
  if (fMutex) {fMutex->UnLock(); delete fMutex; fMutex = NULL;}
  return iret;
}

Int_t  TCondition::TimedWait(ULong_t secs, ULong_t nanoSec)
{
  Int_t iret;
  if (fMutex) fMutex->Lock();
  iret = fConditionImp->TimedWait(secs, nanoSec);
  if (fMutex) {fMutex->UnLock(); delete fMutex; fMutex = NULL;}
  return iret;
}
