//*CMZ :  2.00/00 04/11/97  09.59.07  by  Victor Perev
//*CMZ :  1.03/03 25/08/97  16.25.33  by  Fons Rademakers
//*-- Author :    Fons Rademakers   25/06/97

//*KEEP,CopyRight,T=C.
/*************************************************************************
 * Copyright(c) 1995-1999, The ROOT System, All rights reserved.         *
 * Authors: Rene Brun and Fons Rademakers.                               *
 *                                                                       *
 * For the licensing terms see $ROOTSYS/AA_LICENSE.                      *
 * For the list of contributors see $ROOTSYS/AA_CREDITS.                 *
 *************************************************************************/
//*KEND.

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// TPosixMutex                                                          //
//                                                                      //
// This class provides an interface to the posix mutex routines.        //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

//*KEEP,TThread,T=C++.
#include "TThread.h"
//*KEEP,TPosixMutex,T=C++,IF=POSIX.
#include "TPosixMutex.h"
//*KEEP,PosixThreadInc,T=C++,IF=POSIX.
#include "PosixThreadInc.h"
//*KEND.

ClassImp(TPosixMutex)
//______________________________________________________________________________
TPosixMutex::TPosixMutex()
{
   int rc;
// Create a posix mutex lock.
   rc = ERRNO(pthread_mutex_init(&fMutex, 0));
   if (rc != 0)
      ::SysError("TMutex::TMutex", "pthread_mutex_init error");
   rc = ERRNO(pthread_mutex_init(&fGuard, 0));
   if (rc != 0)
      ::SysError("TMutex::TMutex", "pthread_mutex_init error");
}

//______________________________________________________________________________
TPosixMutex::~TPosixMutex()
{
   // TMutex dtor.

   int rc = ERRNO(pthread_mutex_destroy(&fMutex));
   if (rc != 0)
      ::SysError("TMutex::~TMutex", "pthread_mutex_destroy error");
   rc =     ERRNO(pthread_mutex_destroy(&fGuard));
   if (rc != 0)
      ::SysError("TMutex::~TMutex", "pthread_mutex_destroy error");
}

//______________________________________________________________________________
Int_t TPosixMutex::Lock()
{
  int rc,iqwe;
   // Lock the mutex.
   iqwe = ERRNO(pthread_mutex_lock(&fGuard));
   rc   = ERRNO(pthread_mutex_lock(&fMutex));
   iqwe = ERRNO(pthread_mutex_unlock(&fGuard));
return rc;
}

//______________________________________________________________________________
Int_t TPosixMutex::TryLock()
{
   // Lock the mutex.

   return ERRNO(pthread_mutex_trylock(&fMutex));
}

//______________________________________________________________________________
Int_t TPosixMutex::UnLock(void)
{
// Unlock the mutex.
    return ERRNO(pthread_mutex_unlock(&fMutex));
}
